Package: cummeRbund
Title: Analysis, exploration, manipulation, and visualization of
        Cufflinks high-throughput sequencing data.
Version: 2.4.1
Date: 2012-05-04
Author: L. Goff, C. Trapnell, D. Kelley
Description: Allows for persistent storage, access, exploration, and manipulation of Cufflinks high-throughput sequencing data.  In addition, provides numerous plotting functions for commonly used visualizations.
Imports: methods, plyr, BiocGenerics
Depends: R (>= 2.7.0), BiocGenerics (>= 0.3.2), RSQLite, ggplot2,
        reshape2, fastcluster, rtracklayer, Gviz
Suggests: cluster, plyr
Maintainer: Loyal A. Goff <lgoff@csail.mit.edu>
License: Artistic-2.0
Collate: AllGenerics.R AllClasses.R database-setup.R methods-CuffSet.R
        methods-CuffData.R methods-CuffDist.R methods-CuffGeneSet.R
        methods-CuffFeatureSet.R methods-CuffGene.R
        methods-CuffFeature.R tools.R
LazyLoad: yes
biocViews: HighThroughputSequencing, HighThroughputSequencingData,
        RNAseq, RNAseqData, GeneExpression, DifferentialExpression,
        Infrastructure, DataImport, DataRepresentation, Visualization,
        Bioinformatics, Clustering, MultipleComparisons, QualityControl
Packaged: 2013-12-13 05:38:16 UTC; biocbuild
