#########################################################################
#########################################################################
#																		#
# cummeRbund: An R package for exploration, analysis, manipulation, 	#
# and visualization of Cufflinks high-throughput RNA Sequencing data.	#
#																		#
#########################################################################
#########################################################################

Authors:
	- Loyal A. Goff (1,2)
	- Cole Trapnell (2)
	
Maintainer:
	- Loyal A. Goff <lgoff@csail.mit.edu>

Creative Contributions & Support:
	- John L. Rinn (2)
	- Manolis Kellis (1)
	- Ron Hart (3)	
	- David Hendrickson (2)

Affiliations:
	(1) Computer Science and Artificial Intelligence Laboratory, Massachusetts Institute of Technology, Cambridge, MA, USA
	(2) Stem Cells and Regenerative Medicine, Harvard University, Cambridge, MA, USA
	(3) Rutgers Stem Cell Research Center, Rutgers University, New Brunswick, NJ, USA
