Run Report
==========

Run Parameters
-------------------
```{r}
alpha=0.05
runInfo()
```


Global Statistics
-----------------
```{r}
cuff
```

### FPKM distributions
```{r fig.width=7, fig.height=6}
csDensity(genes(cuff))
csDensity(isoforms(cuff))
```

### Scatterplots
```{r}
csScatterMatrix(genes(cuff),logMode=T)
```

Replicate Information
---------------------


Model fit
---------

Differential Expression Analysis
--------------------------------

### Significant Genes

```{r fig.width=6,fit.height=6}
sigMatrix(cuff,level="genes")
```

```{r}
csVolcanoMatrix(genes(cuff),alpha=alpha)
```

```{r}
sigGeneIDs<-getSig(cuff,alpha=alpha)
sigGenes<-getGenes(cuff,sigGeneIDs)
```
There are `r length(sigGeneIDs)` significantly different genes at a `r alpha*100`$\%$ FDR. 

#### Visualizations
```{r fig.width=6,fit.height=10}
csHeatmap(sigGenes,cluster="both",labRow=F)
```

### Significant Isoforms
```{r fig.width=6,fit.height=6}
sigMatrix(cuff,level="isoforms")
```